""" This file contains the logic for the IDGAF server. It can be used to make a stand-alone 
    IDGAF server, or it can be used as a module in a larger application. 
"""
from datetime import datetime
import json
import random
import requests
import settings
from const import OPENAI_TIMEOUT

URL = "https://ai.hackedyour.info/api/chat/completions"
""" The URL for the OpenAI API. """

DATA = {"model": settings.OPEN_AI_COMPLETION_MODEL,
        "messages": [{"role":"system", "content":"You are AIDGAF server. You tell about how much people dont give a fuck"},{"role":"user", "content": settings.PROMPTS[0] }],
        "temperature": settings.TEMPERATURE,
        "max_tokens": settings.OPEN_AI_MAX_TOKENS
        }
""" The data to send to OpenAI. """

request_headers = {"Authorization": "Bearer " +
                   settings.APIKEY, "Content-Type": "application/json"}
""" The headers to send to OpenAI. """


def get_response_base_object(text: str) -> dict:
    """ This is used to create the response object for the server.
        Parameters:
            text: The text to return to the client.
        Returns:
            A dictionary containing the response object.
    """
    result_object = {}
    result_object["message"] = {}
    result_object["service"] = "AIDGAF Server"
    result_object["message"]["data"] = {}
    result_object["message"]["data"]["resultObject"] = text
    result_object["timestamp"] = datetime.utcnow().timestamp()
    return result_object


def parse_idgaf_request(idgaf_command) -> [int, dict]:
    """ This function handles the IDGAF command. It will return a response object.
        Parameters:
            command: The command object received from the client.
        Returns:
            A tuple containing the status code and the response object.
    """
    the_data = get_prompt(idgaf_command)
    response = get_gpt_response(the_data)
    try:
        response_text = response.json()['choices'][0]['message']["content"].strip()

    except KeyError:
        response_text = response.text
    print(response_text)
    obj = get_response_base_object(response_text)
    return [response.status_code, obj]


def get_gpt_response(data) -> requests.Response:
    """ This function communicates with OpenAI and returns a response object.
        Parameters:
            data: The data to send to OpenAI.
        Returns:
            The response object from OpenAI.
    """
    gpt_response = requests.post(
        url=URL, json=data, headers=request_headers, timeout=OPENAI_TIMEOUT)
    return gpt_response


def get_prompt(command) -> dict:
    """ Selects a prompt from the PROMPTS list and replaces the USERNAME placeholder 
        with the username.
        Parameters:
            command: The command object received from the client.
        Returns:
            A dictionary containing the data to send to OpenAI.
    """
    replyTo=command['message']['data'].get('replyTo',"")
    replyText=command['message']['data'].get('replyText',"")
    inputText=command['message']['data'].get('inputText',"")
    my_prompt = random.choice(settings.PROMPTS)
    my_prompt = my_prompt.replace(
        "USERNAME", command['message']['data']['username'])
    if replyTo:
        my_prompt=replyTo +"said \""+replyText+".\"\n In response, "+my_prompt
    if inputText:
        my_prompt="With the following in mind: "+ command['message']['data']['username'] +" doesn't care about \""+inputText+"\".\n\n"+my_prompt
    
    print(my_prompt)
    the_data = DATA
    the_data["messages"][-1]["content"] = my_prompt
    return the_data


# This function is for testing the IDGAF capabilities without a server.
if __name__ == "__main__":
    INPUT = '''{"service":"papa","message":
    {"command":"aidgaf","data":{"username":"AdamOutler"},
    "timestamp":1675725191},
    "hash":"1bc73914478835d03f9ebdfb46328321d2bb656647e2876d6f162cc1860607fcfca8d825c48e390a6a254ee0835c8a4fe5f9a25795a3a0880ae5a23e9c132cf2"}'''
    test_command = json.loads(INPUT)
    [code, result] = parse_idgaf_request(test_command)
    print(result)
